function [frame_bandwidth] = bandwidth(frame_payload, ack)
% BANDWIDTH required bandwidth for each link
%
% Parameters:
%   frame_payload    [bits] frame payload (data) 
%   ack     [true/false] retransmission (packet with acknowledgment)?
%
%   frame_bandwidth  [bits] required bandwidth for frame (overhead + frame payload)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global aMaxPHYPacketSize aMaxSIFSFrameSize macAckWaitDuration macMaxFrameRetries;

%MPDU = MHR + MSDU + MFR

PHR = 48; %[bits] PHY header
MHR = 72; %[bits] MAC header - addressing mode 0x02 (16-bit addresses)
MFR = 16; %[bits] MAC footer (FCS)
NHR = 64; %[bits] NWK header
SIFS = 48; %[bits] Short inter-frame spacing
LIFS = 160; %[bits] Long inter-frame spacing
ack_length = 40; %[bits] length of ACK frame

MPDU_overhead = MHR + MFR + NHR; %[bits] MAC Protocol Data Unit (MPDU) overhead
frame_payload_max = aMaxPHYPacketSize-MPDU_overhead; %maximal available frame payload

if frame_payload > frame_payload_max %more frames
    frame_nbr = frame_payload/frame_payload_max; %number of frames
    if (frame_nbr-floor(frame_nbr))>0
        payload_rest = frame_payload - floor(frame_nbr)*frame_payload_max; %a remainder of integral number of frames
        if (payload_rest + MPDU_overhead) > aMaxSIFSFrameSize
            IFS = LIFS;
        else
            IFS = SIFS;
        end

        if ack
            frame_bandwidth = floor(frame_nbr)*((PHR + aMaxPHYPacketSize + macAckWaitDuration)*(macMaxFrameRetries + 1) + LIFS) + ((PHR + MPDU_overhead + payload_rest + macAckWaitDuration)*(macMaxFrameRetries + 1) + IFS);
        else %no ACK
            frame_bandwidth = floor(frame_nbr)*(PHR + aMaxPHYPacketSize + LIFS) + (PHR + MPDU_overhead + payload_rest + IFS);
        end
    else
        if ack
            frame_bandwidth = frame_nbr*((PHR + aMaxPHYPacketSize + macAckWaitDuration)*(macMaxFrameRetries + 1) + LIFS);
        else %no ACK
            frame_bandwidth = frame_nbr*(PHR + aMaxPHYPacketSize + LIFS);
        end        
    end    
else %single frame
    if (MPDU_overhead + frame_payload) > aMaxSIFSFrameSize
        IFS = LIFS;
    else
        IFS = SIFS;
    end

    if ack
        frame_bandwidth = (PHR + MPDU_overhead + frame_payload + macAckWaitDuration)*(macMaxFrameRetries + 1) + IFS;
    else
        frame_bandwidth = PHR + MPDU_overhead + frame_payload + IFS;
    end    
end


