function [U,P]=floyd(A)
% FLOYD The Floyd-Warshall algorithm for finding shortest paths in a
% weighted, directed graph.
% 
% Parameters:
%   A   adjacency matrix (Aij = weight of a oriented edge between vertices
%   i and j)
%
%   U   matrix of the shortest path (Uij = length of a path from i to j)
%   P   matrix of the vertex predecessors on the shortest path (Pij  = the
%   predecessor of j on the shortest path from i to j)

% example of an adjacency matrix A:
% A =[0,1,1,Inf,Inf,Inf,Inf;
%   1,0,Inf,1,1,Inf,Inf;
%   1,Inf,0,Inf,Inf,1,Inf;
%   Inf,1,Inf,0,Inf,Inf,Inf;
%   Inf,1,Inf,Inf,0,Inf,Inf;
%   Inf,Inf,Inf,Inf,Inf,0,Inf;
%   Inf,Inf,1,Inf,Inf,Inf,0];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = length(A);
P = zeros(n,n);

for i=1:n
    for j=1:n
        if((i~=j)&&(A(i,j)~=Inf)&&(A(i,j)~=0))
            P(i,j)=i; 
        end
    end
end

%calculate the shortest paths
U=A;
for k=1:n           %k-th vertex
    for i=1:n       %row
        for j=1:n   %column
            if(U(i,j)>U(i,k)+U(k,j))
                U(i,j)=U(i,k)+U(k,j);   %change the length
                P(i,j)=P(k,j);  %change the predecessor
            end
        end
    end
end

            
        