function [xmin,q,solutionStatus,noIntegerVar] = ilppcplexreadres_zig(filenameSolutionIn,filenameLPModel,n)
%The function reads file produced by CPLEX solver and returns result of
%optimization, i.e. vector with optimal solution. The vector must be
%contained in the generated file in the following form:


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Examples of input files (consisting results)

% Time limit exceeded, integer feasible:  Objective =   8.6000000000e+001
% Current MIP best bound =   3.1247704124e+001 (gap = 54.7523, 63.67%)
% Solution time =    1.02 sec.  Iterations = 3900  Nodes = 492 (350)
% 
% CPLEX> Variable Name           Solution Value
% Cmax                         86.000000
% s(1)                         52.000000


% Integer optimal solution:  Objective =   4.1000000000e+001
% Solution time =    0.16 sec.  Iterations = 1603  Nodes = 499
% 
% CPLEX> Variable Name           Solution Value
% Cmax                         41.000000
% s(1)                         13.000000

% Time limit exceeded, no integer solution.
% Current MIP best bound =   5.9000000000e+001 (gap is infinite)
% Solution time =    1.02 sec.  Iterations = 1064  Nodes = 66 (38)
% 
% CPLEX> No integer feasible solution exists.


% Integer infeasible or unbounded.
% Current MIP best bound is infinite.
% Solution time =    0.02 sec.  Iterations = 16  Nodes = 0
% 
% CPLEX> No integer feasible solution exists.
% CPLEX> 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Read solution status

solutionStatus = 0;     %Status of the solution 0=nonfeasible, 1=optimal, 2=feasible but not optimal

fid = fopen(filenameSolutionIn,'r');
if(fid == -1)
    error('GPR:ilppcplexreadres',['Can not read file ''' filename '''']);
end

xmin = [];
q = {};
noIntegerVar = 0;

while 1
    tline = fgetl(fid);
    if ~ischar(tline), break, end
    if(~isempty(strfind(tline,'Integer infeasible or unbounded.')))
        fclose(fid);
        return;
    end
    if(~isempty(strfind(tline,'Integer infeasible')))
        fclose(fid);
        return;
    end
    if(~isempty(strfind(tline,'Presolve - Infeasible.')))
        fclose(fid);
        return;
    end
    if(~isempty(strfind(tline,'Presolve - Unbounded or infeasible.')))
        fclose(fid);
        return;
    end
    if(~isempty(strfind(tline,'Time limit exceeded, no integer solution.')))
        fclose(fid);
        return;
    end
    if(~isempty(strfind(tline,'Time limit exceeded, integer feasible:')))
        solutionStatus = 2;
        break;
    end
    if(~isempty(strfind(tline,'Integer optimal solution:')))
        solutionStatus = 1;
        break;
    end
    if(~isempty(strfind(tline,'MIP - Integer optimal solution:')))  %version 11.0
        solutionStatus = 1;
        break;
    end
    if(~isempty(strfind(tline,'MIP - Integer optimal:')))  %version 11.0
        solutionStatus = 1;
        break;
    end
    if(~isempty(strfind(tline,'Dual simplex - Optimal:')))
        solutionStatus = 1;
        break;
    end
end

xmin = zeros(1,n);
%procNotAssignedYet = {};
% for(i=1:n)
%     q{i} = ones(1,length(dedication{i}));
%     procNotAssignedYet{i} = ones(1,length(dedication{i}));
% end
q = zeros(1,n);
startReading = 0;
%modelUseZVariable = 0;
%y = [];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Parse results
while 1
    tline = fgetl(fid);
    if ~ischar(tline), break, end
    
    %Start reading after string 'CPLEX> Variable Name'
    %if(~isempty(strfind(tline,'CPLEX> Variable Name')))
    if(~isempty(strfind(tline,'Variable Name'))) %version 11.0
        startReading = 1;
        continue;
    end

    %Ignore emty command prompts
    if(strcmp(tline,'CPLEX> '))
        break
    end
    
    if(startReading == 0)
        continue;
    end
    
    if(~isempty(strfind(tline,'All other variables')))
        break;
    end
    
    [from to] = regexp(tline, '\S+\s');
    varname = tline(from:(to-1));
    if(isempty(varname))
        fclose(fid);
        error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
    end 
    [from to] = regexp(tline, '\s[\d\.-]+$');
    value = tline(from:to);
    if(isempty(value))
        fclose(fid);
        error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
    end 
    value = str2num(value);
    
    %Read start times
    if(strcmp(varname(1:2),'s('))
        [from to] = regexp(varname, 's\(\d+\)');
        index = varname(from+2:to-1);
        if(isempty(index))
            fclose(fid);
            error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
        end
        index = str2num(index);

        %fprintf('s(%d) = %d\n',index,value);
        xmin(index) = value;
        continue;
    end
    
    %Read q
    if(strcmp(varname(1:2),'q('))
        [from to] = regexp(varname, 'q\(\d+\)');
        index = varname(from+2:to-1);
        if(isempty(index))
            fclose(fid);
            error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
        end
        index = str2num(index);

        %fprintf('s(%d) = %d\n',index,value);
        q(index) = value;
        continue;
    end
    
    
%     %Read processor assignment from z
%     if(strcmp(varname(1:2),'z('))
%         modelUseZVariable = 1;
%         %Parse task name
%         [from to] = regexp(varname, 'z\(\d+,');
%         task = varname(from+2:to-1);
%         if(isempty(task))
%             fclose(fid);
%             error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
%         end
%         task = str2num(task);
% 
%         %Parse processor index
%         [from to] = regexp(varname, ',\d+,');
%         processor = varname(from+1:to-1);
%         if(isempty(processor))
%             fclose(fid);
%             error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
%         end
%         processor = str2num(processor);
% 
%         %Parse processor group index
%         [from to] = regexp(varname, ',\d+\)');
%         group = varname(from+1:to-1);
%         if(isempty(group))
%             fclose(fid);
%             error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
%         end
%         group = str2num(group);
% 
%         %fprintf('z(%d,%d,%d) = %d\n',task,processor,group,value);
%         if(value==1)
%             paVector = q{task};
%             pnayVector = procNotAssignedYet{task};
%             procIndex = find(dedication{task}==group & pnayVector==1);
%             procIndex = procIndex(1);
%             paVector(procIndex) = processor;
%             pnayVector(procIndex)=0;
%             procNotAssignedYet{task} = pnayVector;
%             q{task} = paVector;
%         end
%         continue;
%     end
% 
%     %Read processor assignment from y
%     if(strcmp(varname(1:2),'y('))
%         %Parse task name
%         [from to] = regexp(varname, 'y\(\d+,');
%         taskI = varname(from+2:to-1);
%         if(isempty(taskI))
%             fclose(fid);
%             error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
%         end
%         taskI = str2num(taskI);
% 
%         %Parse processor index
%         [from to] = regexp(varname, ',\d+\)');
%         taskJ = varname(from+1:to-1);
%         if(isempty(taskJ))
%             fclose(fid);
%             error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
%         end
%         taskJ = str2num(taskJ);
% 
%         %fprintf('y(%d,%d) = %d\n',taskI,taskJ,value);
%         if(value~=0)
%             y(taskI,taskJ) = value;
%         end
%         continue;
%     end


end

if(startReading == 0)
    fclose(fid);
    error('GPR:ilppcplexreadres',['Model processing error. See ''' filenameSolutionIn '''.']);
end


% if(modelUseZVariable == 0)
%     if(size(y,1)<n || size(y,2)<n)
%         y(n,n) = 0;
%     end
%     y = y + y';
%     q = y;
% end

fclose(fid);

%Count number of integer variables
fid = fopen(filenameLPModel,'r');
count = 0;
noIntegerVar = 0;

while(1)
    line = fgetl(fid);
    if(line == -1)
        break;
    end
    if(strcmp(line,'Generals'))
        count = 1;
        continue;
    end
    
    if(strcmp(line,''))
        count = 0;
        continue;
    end
    
    if(count == 1)
        noIntegerVar = noIntegerVar + 1;
    end
        
end

fclose(fid);

%end of file
