function [p,W,A,dedication,proc_names,task_names]=intree2tasks(p,W,A,dedication,proc_names,task_names,mytree,cp,no_clusters)
%creates tasks from the intree structure and adds them to current tasks
%first solver processor is considered to be dummy processsor for dummy tasks
%
% PARAMETERS:
%   p               processing time
%   A               adjacency matrix constraining ofset start times
%   W               adjacency matrix constraining real start times               
%   dedication      dedication to given processors
%   proc_names      names of processors corresponds with routers' names
%   task_names      task_name in sequence
%   mytree
%   cp
%   no_clusters
%
% Copyright Zdenek Hanzalek, hanzalek@fel.cvut.cz 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%initialization
task_id = zeros(1,length(mytree.father));
mark = zeros(1,length(mytree.father));    %mark(i) = 1 beeing inserted, >=2 was inserted, =0 was not inserted yet
root_id = find(mytree.father==0); %root of tree == sink of data flow
mark(root_id) = 1; 

%starting from the root create tasks
while any(mark==1) 
    next_mark = zeros(1,length(mytree.father));
    for node_id = find(mark==1)  %for each node which is beeing inserted
        %create new task
        task_names = [task_names;mytree.task_names(1:3,node_id)'];
        task_id(node_id) = size(task_names,1); %index of the task
        dedication{task_id(node_id)} = [1]; %dummy processor
        %p(task_id(node_id)) = mytree.processing_time(node_id);
        p(task_id(node_id)) = 0;
        W = [W,-Inf*ones(size(W,1),1);-Inf*ones(1,size(W,1)+1)];    %add empty column and row
        A = [A,-Inf*ones(size(A,1),1);-Inf*ones(1,size(A,1)+1)];    %add empty column and row
        %everything is related to the start of the block
        %release date - join it to the dummy task
        W(1,task_id(node_id)) = max(0,mytree.r(node_id)-cp(1,node_id));   
        %relative deadline
        if mytree.d_rel(node_id)~=Inf
            W(task_id(root_id),task_id(node_id)) = -(mytree.d_rel(node_id)+cp(1,node_id)+cp(2,node_id)*mytree.rld(node_id)-(cp(1,root_id)+cp(2,root_id)+cp(3,root_id)*(1-mytree.rld(root_id))));
        end
        if node_id==root_id
            %abs deadline
            if mytree.d_abs~=Inf
                W(task_id(node_id),1) = -(mytree.d_abs-(cp(1,node_id)+cp(2,node_id)+cp(3,node_id)));
            end
        else
            %precedence relation - forward edge
            W(task_id(node_id),task_id(mytree.father(node_id))) = cp(1,node_id)+cp(2,node_id)+cp(3,node_id);
        end
        %join new task to its cluster
        for i=2:no_clusters
            if task_names(i,:)==[node_id,node_id,node_id]
                A(i,task_id(node_id))=0;   
                A(task_id(node_id),i)=0;
                break
            end
        end
        %change mark
        sons = find(mytree.father==node_id);
        next_mark(sons) = ones(1,length(sons));
    end
    %go to next level
    mark = next_mark+2*mark;
end
active = find(mytree.father~=-Inf);
if active~=find(mark~=0)
    error('zigbee_tree:intree2tasks','Inconsistent set of nodes participating in multicast');
end

